% global_classifier.{ fft_params.{ fs, window_size, band_boundaries, overlapping_windows }, 
%                     tree }
% search_params.{ window_size, window_step }
% individual_results.{labels, confidences, passed}
function [error_rate, threshold, individual_results] = test_clips(data_dir, pos_clip_list, neg_clip_list, num_synth, num_neg, global_classifiers, output_dir)
    
    fraction_of_pos_list_to_use = 0.5;            
    
    num_bands = length(global_classifiers(1).fft_params.band_boundaries);
    num_features = 138;
    
    num_stages = length(global_classifiers);
    
    neg_clips = read_wav_files(data_dir, neg_clip_list);
    pos_clips = read_wav_files(data_dir, pos_clip_list);
    
    pos_clips = pos_clips(round(length(pos_clips)*fraction_of_pos_list_to_use)+1:end);    
    
    % data(num_times, num_samples) 
    % by convention, all positive examples are first
    % samples have no border
    
    med_length = global_classifiers(1).data_length;    
    
    [data, labels] = create_uniform_length_clips(pos_clips, neg_clips, num_synth, num_neg, med_length);
        
    num_samples = size(data, 2);    
    num_pos = sum(labels == 1);
    num_neg = num_samples - num_pos;
    
    features = zeros(num_features, num_samples);
    for s = 1:num_samples
        features(:, s) = extract_global_features_window(data(:, s), global_classifiers(1).fft_params);
    end    
    features = transpose(features);    
    
    data_indices = (1:length(labels));
    
    for stage = 1:num_stages

        if stage > 1
            features = features(passed_indices, :);            
%             if stage == 2
%                 features = [features confidences];
%             else
%                 features(:, end) = confidences;
%             end
        end                
        
        confidences = zeros(size(features, 1), 1);    
        num_trees = length(global_classifiers(stage).wcs);

        for t = 1:num_trees            
            [class_labels, node_ids] = treeval(global_classifiers(stage).wcs(t).tree, features);
            for s = 1:length(node_ids)
                confidences(s) = confidences(s) + global_classifiers(stage).wcs(t).node_confidences(node_ids(s));
            end
        end            
        
        if stage == num_stages
            passed_labels = labels(data_indices);
            pos_cost = 1/(2*sum(passed_labels==1));
            neg_cost = 1/(2*sum(passed_labels==-1));
            max_pos_error = 0.05+ 0.05*(num_stages==1);
            threshold = get_best_threshold(confidences, passed_labels, pos_cost, neg_cost, max_pos_error);
            disp(['chosen threshold = ' num2str(threshold)]);
        else
            threshold = global_classifiers(stage).threshold;
        end
        
        % get passed examples
        passed_indices = zeros(length(class_labels), 1);
        num_passed = 0;
        for s = 1:length(class_labels)
            if confidences(s) > threshold
                num_passed = num_passed + 1;
                passed_indices(num_passed) = s;
            end
        end
        passed_indices = passed_indices(1:num_passed);
        
        data_indices = data_indices(passed_indices);        
        passed_labels = labels(data_indices);
        confidences = confidences(passed_indices);                
        
        num_pos_passed = sum(passed_labels == 1);
        num_neg_passed = sum(passed_labels == -1);
        pos_error_rate = (num_pos-num_pos_passed)/num_pos;
        neg_error_rate = num_neg_passed/num_neg;
        
        disp(['pos_errors = ' num2str(pos_error_rate) '  neg_errors = ' num2str(neg_error_rate)])
        
    end        
    
    passed_labels = labels(data_indices);        
    disp(['num passed = ' num2str(length(passed_labels))]);
    
    num_pos_passed = sum(passed_labels == 1);
    num_neg_passed = sum(passed_labels == -1);
    pos_error_rate = (num_pos-num_pos_passed)/num_pos;
    neg_error_rate = num_neg_passed/num_neg;

    error_rate = pos_error_rate/2 + neg_error_rate/2;
    
    disp(['pos_errors = ' num2str(pos_error_rate) '  neg_errors = ' num2str(neg_error_rate) ...
          '  all_errors = ' num2str(error_rate)]);
        
    % write out errors to file      
    assigned_labels = -1*ones(length(labels), 1);
    assigned_labels(data_indices) = 1;
    errors = (assigned_labels ~= labels);          
    for s = 1:length(errors)
        if errors(s) & labels(s) == 1
            wavwrite(data(:, s), [output_dir '/pos_failure' num2str(s) '.wav']);
        elseif errors(s)
            wavwrite(data(:, s), [output_dir '/neg_failure' num2str(s) '.wav']);
        end
    end
    
    individual_results.labels = labels;
    individual_results.passed = zeros(length(labels), 1);
    individual_results.passed(data_indices) = 1;
    individual_results.confidences = zeros(length(labels), 1);
    individual_results.confidences(data_indices) = confidences;
