function wcs = compute_decision_tree(features, labels, num_trees)
% Learn a boosted decision tree classifier
%
% INPUT:
% features(num_features, num_samples): the data 
% labels: the true classification labels
% num_trees: the minimum number of trees to include in classifier; the
%            actual number of trees num_trees < t < num_trees+5, depending on the
%            training error
% 
% OUTPUT:
% wcs(ntrees): the weak classifers


X = transpose(features);

if 1
    y = repmat('p', length(labels), 1);
    for i = 1:length(labels)
        if labels(i) == -1
            y(i) = 'n';
        end
    end
else
    y = labels;
end

num_pos = sum(labels == 1);
num_neg = sum(labels == -1);
num_samples = num_pos + num_neg;    

total_confidences = zeros(num_samples, 1);
prod_Z = 1;

pos_weight = 1/(2*num_pos);
neg_weight = 1/(2*num_neg);
w = [pos_weight*ones(num_pos, 1) ; neg_weight*ones(num_neg, 1)];

for i = 1:num_trees+5
    
    % limit the weight of any one example to some fraction of the total
    % weight
    disp(['max weight ' num2str(max(w))])
    w = min(w, 50*max(pos_weight, neg_weight));
    w = w / sum(w);
    
    % weighted tree fit
    wcs(i).tree = treefitw(X, y, w, 1/num_samples, 'splitmin', 5/num_samples, 'catidx', [33 34]);

    [error, se_err, nnodes, best_level] = treetestw(wcs(i).tree, 'crossvalidate', X, y, w, 50, 'nsamples', 5);        
    
    best_level = max([best_level (max(wcs(i).tree.prunelist)-7)]);
    disp(['cv error = ', num2str(error(best_level+1))])
    %treedisp(wcs(i).tree, 'prunelevel', best_level);treedisp(wcs(i).tree, 'prunelevel', best_level);
    
    wcs(i).tree = treeprune(wcs(i).tree, 'level', best_level);
    
    [class_labels, node_ids] = treeval(wcs(i).tree, X);
    
    confidences = zeros(num_samples, 1);
    wcs(i).node_confidences = zeros(wcs(i).tree.node(end), 1);
    for n = 1:length(wcs(i).node_confidences)
        class_counts = wcs(i).tree.classcount(n, :);
        wcs(i).node_confidences(n) = 0.5*log(class_counts(1)/class_counts(2));
    end            
    for s = 1:num_samples            
        confidences(s) = wcs(i).node_confidences(node_ids(s));
    end
    % compute best alpha (minimize Z)
    %alpha = fminbnd(@(x) compute_expected_confidence(x, labels, confidences, w), 0.001, 1.0);
    alpha = 1.0;
    
    % for debugging
    if 1 
        Z = sum(exp(-alpha*labels.*confidences).*w);
        prod_Z = prod_Z * Z;
        disp(['Z = ', num2str(Z) '  prod_Z = ', num2str(prod_Z)]);        
    end
    total_confidences = total_confidences + confidences*alpha;
    training_error = mean( (total_confidences > 0)*2-1 ~= labels);
    disp(['training error = ' num2str(training_error)])

    
    % compute new weights
    w = reweight_data(labels, confidences, w, alpha);
                
    % for debugging                
    if 0
        training_error = treetest(wcs(i).tree, 'test', X, y);   
        disp(['training error = ', num2str(training_error(best_level))]);             
    end
    disp(['best level ' num2str(best_level)])
    
    if i >= num_trees & training_error < 0.001
        break;
    end
    
end